#include <stdio.h>

typedef struct {
   const unsigned numX, numY, numZ;
   const double *xVec, *yVec, *zVec, *data;
   unsigned xIdx, yIdx, zIdx;
} plecsLUT;

double plecsmax(double a, double b)
{
   if (a > b)
      return a;
   return b;
}

double plecsmin(double a, double b)
{
   if (a < b)
      return a;
   return b;
}

void findIndex(const double *xVec, unsigned numX, double x,
               unsigned* idx, double* fraction)
{
   while (*idx+2 < numX
          && (x > xVec[*idx+1] || (x == xVec[*idx+1] && x == xVec[*idx+2])))
      ++*idx;
   while (*idx > 0
          && (x < xVec[*idx] || (x == xVec[*idx] && x == xVec[*idx-1])))
      --*idx;
   
   /* check for multiple matching x values */
   if (xVec[*idx] == xVec[*idx+1])
   {
      /* check for three matching x values */
      if (*idx > 0 && x == xVec[*idx-1])
      {
         /* choose middle point */
         *fraction = 0.0;
         return;
      }
      /* 2nd check for three matching values */
      if (*idx+2 < numX && x == xVec[*idx+2])
      {
         /* choose middle point */
         ++*idx;
         *fraction = 0.0;
         return;
      }
      /* check for right plane */
      if (x > 0)
      {
         /* choose left point */
         *fraction = 0.0;
         return;
      }
      /* check for left plane */
      if (x < 0)
      {
         /* choose right point */
         *fraction = 1.0;
         return;
      }
      
      /* default: two matching zero values
       * choose mean value */
      *fraction = 0.5;
      return;
   }

   /* default:
    * calculate fraction from the bracketing values */
   *fraction = (x-xVec[*idx])/(xVec[*idx+1]-xVec[*idx]);
}

double plecslu1d(plecsLUT *lut, double x)
{
   double xFrac = 0.;

   findIndex(lut->xVec, lut->numX, x, &lut->xIdx, &xFrac);

   return (1-xFrac)*lut->data[lut->xIdx] + xFrac*lut->data[lut->xIdx+1];
}

#define DATA2D(x,y) lut->data[(lut->xIdx+(x))*lut->numY+lut->yIdx+(y)]

double plecslu2d(plecsLUT *lut, double x, double y)
{
   double xFrac = 0., yFrac = 0.;
   double outX1, outX2;

   findIndex(lut->xVec, lut->numX, x, &lut->xIdx, &xFrac);
   findIndex(lut->yVec, lut->numY, y, &lut->yIdx, &yFrac);

   outX1 = (1-yFrac)*DATA2D(0,0)
         +     yFrac*DATA2D(0,1);
   outX2 = (1-yFrac)*DATA2D(1,0)
         +     yFrac*DATA2D(1,1);

   return (1-xFrac)*outX1 + xFrac*outX2;
}

#define DATA3D(x,y,z) lut->data[((lut->xIdx+(x))*lut->numY+(lut->yIdx+(y)))*lut->numZ+lut->zIdx+(z)]

double plecslu3d(plecsLUT *lut, double x, double y, double z)
{
   double xFrac = 0., yFrac = 0., zFrac = 0.;
   double outX1Y1, outX2Y1, outX1Y2, outX2Y2;
   double outX1, outX2;

   findIndex(lut->xVec, lut->numX, x, &lut->xIdx, &xFrac);
   findIndex(lut->yVec, lut->numY, y, &lut->yIdx, &yFrac);
   findIndex(lut->zVec, lut->numZ, z, &lut->zIdx, &zFrac);

   outX1Y1 = (1-zFrac)*DATA3D(0,0,0)
           +     zFrac*DATA3D(0,0,1);
   outX2Y1 = (1-zFrac)*DATA3D(1,0,0)
           +     zFrac*DATA3D(1,0,1);
   outX1Y2 = (1-zFrac)*DATA3D(0,1,0)
           +     zFrac*DATA3D(0,1,1);
   outX2Y2 = (1-zFrac)*DATA3D(1,1,0)
           +     zFrac*DATA3D(1,1,1);

   outX1 = (1-yFrac)*outX1Y1 + yFrac*outX1Y2;
   outX2 = (1-yFrac)*outX2Y1 + yFrac*outX2Y2;

   return (1-xFrac)*outX1 + xFrac*outX2;
}
